@@ -37,7 +37,7 @@ sub _table {
 }
 
 
-sub _name {
+sub _table_name {
     my $self = shift;
     return $self->_table->name;
 }
@@ -55,6 +55,12 @@ sub _columns {
 }
 
 
+sub _column_names {
+    my $self = shift;
+    return $self->_table->column_names_ordered;
+}
+
+
 DESTROY {
     my $self = shift;
     warn "DESTROY $self" if($SQL::DB::DEBUG && $SQL::DB::DEBUG>3);
